/*
    TimeKeeper - A Steampunk Calendar
    Copyright © 2012 Dean Beedell and Harry Whitfield

    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    TimeKeeper - version 1.0b1
    24 April, 2012

    Concept and graphics by Dean Beedell
    dean.beedell@lightquick.co.uk

    Code by Harry Whitfield
    g6auc@arrl.net
*/

/*global earth, moon, wheel, text3, text4, rotator1, rotator2, rotator3, dayOfWeek,
	earthUnderShadow, earthOverShadow, moonUnderShadow, moonOverShadow, monthCogShadow,
	monthCog, cogShadow, cog, wheelShadow, monthCounter, rotatingring, ringtext,
	counterWheel, print
*/

/*properties
    PI, atan2, cos, getDate, getDay, getFullYear, getHours, getMinutes, getMonth, 
    getSeconds, getTime, height, left, length, percent, round, sin, src, text, 
    toLowerCase, top, width
*/

var main_window = {width: 478, height: 478},
	halfWidth   = main_window.width  / 2,
	halfHeight  = main_window.height / 2,
	
	halfEarthWidth  = earth.width  / 2,
	halfEarthHeight = earth.height / 2,
	
	halfEarthUSWidth  = earthUnderShadow.width  / 2,
	halfEarthUSHeight = earthUnderShadow.height / 2,
	
	halfEarthOSWidth  = earthOverShadow.width  / 2,
	halfEarthOSHeight = earthOverShadow.height / 2,
	
	halfMoonWidth  = moon.width  / 2,
	halfMoonHeight = moon.height / 2,
	
	halfMoonUSWidth  = moonUnderShadow.width  / 2,
	halfMoonUSHeight = moonUnderShadow.height / 2,
	
	halfMoonOSWidth  = moonOverShadow.width  / 2,
	halfMoonOSHeight = moonOverShadow.height / 2,

	radToDeg = 180 / Math.PI,
	degToRad = Math.PI / 180,
	rotToPc  = 100 / 360;

//////////////////////////////////////////////////////////////////////////////////////////

function dayAngle(d) {
    var year  = d.getFullYear(),        // 20yy
        month = d.getMonth(),           // 0..11
        day   = d.getDate(),            // 1..31

        daysInMonth = function (month, year) {  // month 0..11
            var monthDays = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];

            if (month !== 1) { return monthDays[month]; }
            if ((year %   4) !== 0) { return 28; }
            if ((year % 400) === 0) { return 29; }
            if ((year % 100) === 0) { return 28; }
            return 29;
        };

    return 360 * (year - 2012) + 30 * month + Math.round(30 * day / daysInMonth(month, year));
}

function weekDayOf(d) {
	var dow = ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"];

	return dow[d.getDay()];
}

function dayOf(d) {
	return d.getDate();
}

function monthOf(d) {
	var months = ["January", "February", "March", "April", "May", "June",
		"July", "August", "September", "October", "November", "December"];

	return months[d.getMonth()];
}

function theYearOf(d) {
	return d.getFullYear() % 100;
}

//////////////////////////////////////////////////////////////////////////////////////////

var date, newDate, lastDow = "", angle0 = dayAngle(new Date()) - 120;
var cos = [], sin = [];

(function makeTrigTables() {
    var i;

    for (i = 0; i < 360; i += 1) {
        cos[i] = Math.cos(degToRad * i);
        sin[i] = Math.sin(degToRad * i);
    }
}());

function displayDate(d, angle0) {
    var angle = dayAngle(d),
        angleMod360 = (angle % 360 + 360) % 360,
        earthAngle = (angleMod360 + 240) % 360,
        angleMod30 = earthAngle % 30,
        dow = weekDayOf(d).toLowerCase(),
        year99,
        sMonth,
        earthRadius = 80,
        moonRadius  = 55,

		year  = d.getFullYear(),
//		month = d.getMonth() + 1,
    	day   = d.getDate();


    function moveEarth(angle) {
        angle = (angle + 220) % 360;

        earth.left = Math.round(halfWidth  + earthRadius * cos[angle] - halfEarthWidth);
        earth.top  = Math.round(halfHeight - earthRadius * sin[angle] - halfEarthHeight);

		earthUnderShadow.left = earth.left + halfEarthWidth  - halfEarthUSWidth;
		earthUnderShadow.top  = earth.top  + halfEarthHeight - halfEarthUSHeight;

		earthOverShadow.left  = earth.left + halfEarthWidth  - halfEarthOSWidth;
		earthOverShadow.top   = earth.top  + halfEarthHeight - halfEarthOSHeight;

		earthOverShadow.percent = rotToPc * ((720 - angle - 90) % 360);
    }

    function moveMoon(angle) {
        var lunarMonth  = 29.53058917,	// days
            sunX = halfWidth,
            sunY = halfHeight - 10, 	// sun is off-centre
            bearing;

        angle = ((Math.round(angle * 365.2425 / lunarMonth + angle - 15) % 360) + 360) % 360;

        moon.left = Math.round(earth.left + halfEarthWidth  + moonRadius * cos[angle] - halfMoonWidth);
        moon.top  = Math.round(earth.top  + halfEarthHeight - moonRadius * sin[angle] - halfMoonHeight);

		moonUnderShadow.left = moon.left + halfMoonWidth  - halfMoonUSWidth;
		moonUnderShadow.top  = moon.top  + halfMoonHeight - halfMoonUSHeight;

		moonOverShadow.left  = moon.left + halfMoonWidth  - halfMoonOSWidth;
		moonOverShadow.top   = moon.top  + halfMoonHeight - halfMoonOSHeight;

		bearing = radToDeg * Math.atan2(moon.top + halfMoonHeight - sunY, moon.left + halfMoonWidth - sunX);
		moonOverShadow.percent = rotToPc * ((Math.round(bearing - 90) + 720) % 360);
    }

    function pad(s, n) {
    	while (s.length < n) { s = "0" + s; }
    	return s;
    }

    year99  = year % 100;
    sMonth = monthOf(d).toLowerCase();

    rotatingring.percent = rotToPc * (360 - angleMod30);
    ringtext.percent     = rotToPc * (360 - earthAngle);
    
	counterWheel.percent = rotToPc * ((earthAngle + 309) % 360);

    monthCogShadow.percent =
        monthCog.percent = rotToPc * earthAngle;

    cogShadow.percent =
        cog.percent = rotToPc * earthAngle;

    wheelShadow.percent =
		wheel.percent = rotToPc * ((angle0 - earthAngle + 720) % 360);

    dayOfWeek.src = "Days\\" + dow + ".png";
    
//  dayTensCounter.src = "Digits\\" + String(Math.floor(day / 10)) + ".png";
//  dayUnitsCounter.src = "Digits\\" + String(day % 10) + ".png";
    
	text3.text = pad(String(day), 2);

 	monthCounter.src = "Months\\" + sMonth + ".png";

//  yearTensCounter.src  = "Digits\\" + String(Math.floor(year99 / 10)) + ".png";
//  yearUnitsCounter.src = "Digits\\" + String(year99 % 10) + ".png";

 	text4.text = pad(String(year99), 2);

    moveEarth(earthAngle);
    moveMoon(angle);
}

function displayTime(d) {
    var hours = d.getHours() % 12,
        mins  = d.getMinutes(),
        secs  = d.getSeconds();

	rotator3.percent  =  5 * secs / 3;
	rotator2.percent  =  5 * (mins / 3 + secs / 180);
	rotator1.percent  = 25 * (hours / 3 + mins / 180);
}

//////////////////////////////////////////////////////////////////////////////////////////

function timerFired(Sender) {
	var d = new Date(),
		dow = weekDayOf(d).toLowerCase();

	if (lastDow === "") {		// first time
		displayDate(d, angle0);
		date = d;
		lastDow = dow;
	}

	displayTime(d);
}

function moonOnClicked(Sender) {
    var d = new Date();

    angle0 = dayAngle(d) - 120;
    displayDate(d, angle0);
    date = d;

    displayTime(d);
}

function earthOnClicked(Sender) {
    var advance = 1, d;
    d = new Date(date.getTime() + advance * 86400000);

    displayDate(d, angle0);
    date = d;
}

var downAngle, downDate, moveEnabled = false;

function ringMouseDown(Sender, Button, Shift, X, Y) {
    if (!moveEnabled) {
   		downAngle = radToDeg * Math.atan2(halfHeight - Y, halfWidth - X);
    	downDate = date.getTime();
    }
}

function ringMouseMove(Sender, Shift, X, Y, Dx, Dy) {
    var angle, delta, d;

	if (moveEnabled) {
    	angle = radToDeg * Math.atan2(halfHeight - Y, halfWidth - X);
    	delta = Math.round(downAngle - angle);
    	d = new Date(downDate + delta * 86400000);
    	displayDate(d, angle0);
    	newDate = d;
    }
}

function ringMouseUp(Sender, Button, Shift, X, Y) {
    if (moveEnabled) {
		date = newDate;
	}
	moveEnabled = !moveEnabled;
}

//////////////////////////////////////////////////////////////////////////////////////////
